//Lukas Kupka BP_upraveny modul Nano-Sim pro potreby prace
/* -*- Mode:C++; c-file-style:"gnu"; indent-tabs-mode:nil; -*- */
/*
 * Copyright (c) 2012 TELEMATICS LAB, DEE - Politecnico di Bari
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Author: Giuseppe Piro <peppe@giuseppepiro.com>, <g.piro@poliba.it>
 */
#include "ns3/core-module.h"
#include "ns3/network-module.h"
#include "ns3/applications-module.h"
#include "ns3/mobility-module.h"
#include "ns3/config-store-module.h"
#include "ns3/internet-module.h"
#include <iostream>
#include "ns3/global-route-manager.h"
#include "ns3/nano-mac-entity.h"
#include "ns3/nano-mac-queue.h"
#include "ns3/nano-spectrum-channel.h"
#include "ns3/nano-spectrum-phy.h"
#include "ns3/nano-spectrum-signal-parameters.h"
#include "ns3/nano-helper.h"
#include "ns3/nano-spectrum-value-helper.h"
#include "ns3/simple-nano-device.h"
#include "ns3/nanointerface-nano-device.h"
#include "ns3/nanorouter-nano-device.h"
#include "ns3/nanonode-nano-device.h"
#include "ns3/backoff-based-nano-mac-entity.h"
#include "ns3/seq-ts-header.h"
#include "ns3/ts-ook-based-nano-spectrum-phy.h"
#include "ns3/mobility-model.h"
#include "ns3/message-process-unit.h"
#include "ns3/transparent-nano-mac-entity.h"
#include "ns3/random-nano-routing-entity.h"
#include "ns3/flooding-nano-routing-entity.h"
NS_LOG_COMPONENT_DEFINE ("health-care");

using namespace ns3;

//pocatecni parametry
double packetInterval = 0.5;
int ROUTINGPROTOCOL = 2; //1->flooding; 2=random;
int MACPROTOCOL = 2; //1->transparent; 2=backoff;

//statistics variables
static int NB_NODES;
static int NB_APPS;
static int SEED;
double DENSITY;
double txrange;
int gate;
int router;

static void PrintPosition(std::ostream *os, std::string foo,
		NetDeviceContainer devs) {
	int num = devs.GetN();
	for (int i = 0; i < num; i++) {
		Ptr<SimpleNanoDevice> d = devs.Get(i)->GetObject<SimpleNanoDevice> ();
		Ptr<MobilityModel> m = d->GetPhy()->GetMobility();
		Vector pos = m->GetPosition();
		*os << Simulator::Now().GetSeconds() << " " << d->GetNode()->GetId()
				<< " " << pos.x << " " << pos.y << " " << pos.z << std::endl;
	}
	Simulator::Schedule(Seconds(0.1), &PrintPosition, os, foo, devs);
}

void PrintMemoryUsage(void) {
	system(
			"ps aux | grep build/scratch/health-care | head -1 | awk '{print $1, $4, $10}'");
}

void Run(int nodes, int APPS);

int main(int argc, char *argv[]) {
	//pocet nano-stroju
	int MIN_N = 600;
	int MAX_N = 600;
	int STEP_N = 200;
	int MIN_SEED = 1;
	int MAX_SEED = 5;
	//pocet nano-rozhrani
	int MIN_GW = 1;
	int MAX_GW = 1;
	int STEP_GW = 1;
	//pocet nano-routeru
	int MIN_R = 21;
	int MAX_R = 21;
	int STEP_R = 2;
	//nastaveni dosahu nano-stroju
	//double txRange[5] = { 0.001, 0.005, 0.01, 0.015, 0.02 };
	double txRange[1] = {0.01};
	int nbTxRanges = 1;

	for (SEED = MIN_SEED; SEED <= MAX_SEED; SEED++) {
		for (gate = MIN_GW; gate <= MAX_GW; gate = gate + STEP_GW) {
			for (router = MIN_R; router <= MAX_R; router = router + STEP_R) {
				for (int i = MIN_N; i <= MAX_N; i = i + STEP_N) {
					for (int j = 0; j < nbTxRanges; j++) {
						txrange = txRange[j];
						NB_NODES = i;
						NB_APPS = i;
						DENSITY = (double) NB_NODES / 300;

						Run(i, NB_APPS);

					}
				}
			}
		}
	}

	return 0;
}

void Run(int nodes, int APPS) {
	std::cout << "START " << NB_NODES << " " << NB_APPS << " DENSITA' "
			<< DENSITY << std::endl;
	//nastaveni delky simulace, dosahu nano-routeru a rozmeru prostoru
	double duration = 3;
	double rtxrange = 0.05;
	double xrange = 1.0;
	double yrange = 0.001;
	double zrange = 0.001;
	//nastaveni parametru vlneni
	double pulseEnergy = 100e-12;
	double pulseDuration = 100;
	double pulseInterval = 10000;
	double powerTransmission = pulseEnergy / pulseDuration;

	Time::SetResolution(Time::FS);

	srand(time(NULL));
	SeedManager::SetSeed(rand());

	UniformVariable random;

	NanoHelper nano;
	//nano.EnableLogComponents ();
	NodeContainer n_routers;
	NodeContainer n_interfaces;
	NodeContainer n_nodes;
	NetDeviceContainer d_interfaces;
	NetDeviceContainer d_nodes;
	NetDeviceContainer d_routers;

	//double start = Simulator::Now ().GetMilliSeconds ();

	n_interfaces.Create(gate);
	d_interfaces = nano.Install(n_interfaces, NanoHelper::nanointerface);
	n_routers.Create(router);
	d_routers = nano.Install(n_routers, NanoHelper::nanorouter);
	n_nodes.Create(nodes);
	d_nodes = nano.Install(n_nodes, NanoHelper::nanonode);
	
	//vytvoreni nano-rozhrani
	for (int x = 0; x < gate; x++) {

		Ptr<NanoInterfaceDevice> interface = d_interfaces.Get(x)->GetObject<
				NanoInterfaceDevice> ();
		Ptr<ConstantPositionMobilityModel> m = CreateObject<
				ConstantPositionMobilityModel> ();

		if (gate == 1) {
			m->SetPosition(Vector(xrange, 0.0, 0.0));
		}
		/*if (gate == 2) {
			m->SetPosition(Vector((double) x / 4 + 0.125, 0.0, 0.0));
		}
		if (gate == 3) {
			m->SetPosition(Vector(x * xrange / (gate - 1), 0.0, 0.0));
		}
		*/
		nano.AddMobility(interface->GetPhy(), m);

		Ptr<NanoRoutingEntity> routing;
		if (ROUTINGPROTOCOL == 1) {
			Ptr<FloodingNanoRoutingEntity> routing2 = CreateObject<
					FloodingNanoRoutingEntity> ();
			routing = routing2;
		} else if (ROUTINGPROTOCOL == 2) {
			Ptr<RandomNanoRoutingEntity> routing2 = CreateObject<
					RandomNanoRoutingEntity> ();
			routing = routing2;
		} else {
		}
		routing->SetDevice(interface);
		interface->SetL3(routing);

		Ptr<NanoMacEntity> mac;
		if (MACPROTOCOL == 1) {
			Ptr<TransparentNanoMacEntity> mac2 = CreateObject<
					TransparentNanoMacEntity> ();
			mac = mac2;
		} else if (MACPROTOCOL == 2) {
			Ptr<BackoffBasedNanoMacEntity> mac2 = CreateObject<
					BackoffBasedNanoMacEntity> ();
			mac = mac2;
		} else {
		}
		mac->SetDevice(interface);
		interface->SetMac(mac);

		interface->GetPhy()->SetTxPower(powerTransmission);
		interface->GetPhy()->SetTransmissionRange(txrange);
		interface->GetPhy()->GetObject<TsOokBasedNanoSpectrumPhy> ()->SetPulseDuration(
				FemtoSeconds(pulseDuration));
		interface->GetPhy()->GetObject<TsOokBasedNanoSpectrumPhy> ()->SetPulseInterval(
				FemtoSeconds(pulseInterval));
		std::cout << "Created NanoInterface " << interface->GetNode()->GetId()
				<< " position " << m->GetPosition().x << " "
				<< m->GetPosition().y << " " << m->GetPosition().z << std::endl;

		interface -> m_txrange = txrange;
		interface -> m_gates = gate;
		interface -> m_routers = router;
		interface -> m_seed = SEED;
		interface -> m_nodes = NB_NODES;
		interface -> m_apps = NB_APPS;

	}

	//vytvoreni nano-routeru
	for (int x = 0; x < router; x++) {

		Ptr<NanoRouterDevice> interface = d_routers.Get(x)->GetObject<
				NanoRouterDevice> ();
		Ptr<ConstantPositionMobilityModel> m = CreateObject<
				ConstantPositionMobilityModel> ();
		m->SetPosition(Vector(x * (xrange / router), 0.0, 0.0)); //nastaveni pozice v ramci prostoru
		nano.AddMobility(interface->GetPhy(), m);

		Ptr<NanoRoutingEntity> routing;
		if (ROUTINGPROTOCOL == 1) {
			Ptr<FloodingNanoRoutingEntity> routing2 = CreateObject<
					FloodingNanoRoutingEntity> ();
			routing = routing2;
		} else if (ROUTINGPROTOCOL == 2) {
			Ptr<RandomNanoRoutingEntity> routing2 = CreateObject<
					RandomNanoRoutingEntity> ();
			routing = routing2;
		} else {
		}
		routing->SetDevice(interface);
		interface->SetL3(routing);

		Ptr<NanoMacEntity> mac;
		if (MACPROTOCOL == 1) {
			Ptr<TransparentNanoMacEntity> mac2 = CreateObject<
					TransparentNanoMacEntity> ();
			mac = mac2;
		} else if (MACPROTOCOL == 2) {
			Ptr<BackoffBasedNanoMacEntity> mac2 = CreateObject<
					BackoffBasedNanoMacEntity> ();
			mac = mac2;
		} else {
		}
		mac->SetDevice(interface);
		interface->SetMac(mac);

		interface->GetPhy()->SetTxPower(powerTransmission);
		interface->GetPhy()->SetTransmissionRange(rtxrange);
		interface->GetPhy()->GetObject<TsOokBasedNanoSpectrumPhy> ()->SetPulseDuration(
				FemtoSeconds(pulseDuration));
		interface->GetPhy()->GetObject<TsOokBasedNanoSpectrumPhy> ()->SetPulseInterval(
				FemtoSeconds(pulseInterval));

		std::cout << "Created NanoRouter " << interface->GetNode()->GetId()
				<< " position " << " position " << m->GetPosition().x << " "
				<< m->GetPosition().y << " " << m->GetPosition().z << std::endl;

		interface -> m_txrange = txrange;
		interface -> m_gates = gate;
		interface -> m_routers = router;
		interface -> m_seed = SEED;
		interface -> m_nodes = NB_NODES;
		interface -> m_apps = NB_APPS;
	}

	//nastaveni mobility modelu
	  MobilityHelper mobility;
	  mobility.SetMobilityModel ("ns3::GaussMarkovMobilityModel",
		"Bounds", BoxValue (Box (0, xrange, 0, yrange, 0, zrange)),
		"TimeStep", TimeValue (Seconds (0.001)),
		"Alpha", DoubleValue (0),
		"MeanVelocity", StringValue ("ns3::UniformRandomVariable[Min=0.35|Max=0.35]"),
		"MeanDirection", StringValue ("ns3::UniformRandomVariable[Min=0|Max=0]"),
		"MeanPitch", StringValue ("ns3::UniformRandomVariable[Min=0.05|Max=0.05]"),
		"NormalVelocity", StringValue ("ns3::NormalRandomVariable[Mean=0.0|Variance=0.0|Bound=0.0]"),
		"NormalDirection", StringValue ("ns3::NormalRandomVariable[Mean=0.0|Variance=0.2|Bound=0.4]"),
		"NormalPitch", StringValue ("ns3::NormalRandomVariable[Mean=0.0|Variance=0.2|Bound=0.4]"));
	  mobility.SetPositionAllocator ("ns3::RandomBoxPositionAllocator",
			  
			  //nasledujici hodnoty nutno upravit dle rozmeru prostoru
		"X", StringValue ("ns3::UniformRandomVariable[Min=0.0|Max=0.7]"),//RandomVariableValue (UniformVariable (0, xrange)),
		"Y", StringValue ("ns3::UniformRandomVariable[Min=0.0|Max=0.001]"),//RandomVariableValue (UniformVariable (0, yrange)),
		"Z", StringValue ("ns3::UniformRandomVariable[Min=0.0|Max=0.001]"));//RandomVariableValue (UniformVariable (0, zrange)));
	  mobility.Install (n_nodes);

	std::string logFile = "nanonodeposition.log";
	std::ofstream os;
	os.open(logFile.c_str());
	Simulator::Schedule(Seconds(0.001), &PrintPosition, &os, logFile, d_nodes);

	//vytvoreni vysilajicich nano-stroju
	for (uint32_t i = 0; i < (d_nodes.GetN()-100); i++) {
	Ptr<ConstantPositionMobilityModel> m = CreateObject<ConstantPositionMobilityModel> ();
	m->SetPosition (Vector (0.0, (0.0 + i/1000), 0.0)); //nastaveni konkrteni pozice, aby byla vzdalenost vsech vysilacu od prijimace stejna
	nano.AddMobility(d_nodes.Get(i)->GetObject<NanoNodeDevice> ()->GetPhy(), m);
	Ptr<NanoNodeDevice> dev = d_nodes.Get(i)->GetObject<NanoNodeDevice> ();
	Ptr<NanoRoutingEntity> routing;
			if (ROUTINGPROTOCOL == 1) {
				Ptr<FloodingNanoRoutingEntity> routing2 = CreateObject<
						FloodingNanoRoutingEntity> ();
				routing = routing2;
			} else if (ROUTINGPROTOCOL == 2) {
				Ptr<RandomNanoRoutingEntity> routing2 = CreateObject<
						RandomNanoRoutingEntity> ();
				routing = routing2;
			} else {
			}
			routing->SetDevice(dev);
			dev->SetL3(routing);
			Ptr<NanoMacEntity> mac;
			if (MACPROTOCOL == 1) {
				Ptr<TransparentNanoMacEntity> mac2 = CreateObject<
						TransparentNanoMacEntity> ();
				mac = mac2;
			} else if (MACPROTOCOL == 2) {
				Ptr<BackoffBasedNanoMacEntity> mac2 = CreateObject<
						BackoffBasedNanoMacEntity> ();
				mac = mac2;
			} else {
			}
			mac->SetDevice(dev);
			dev->SetMac(mac);
			dev->GetPhy()->SetTransmissionRange(txrange);
			dev->GetPhy()->SetTxPower(powerTransmission);
			dev->GetPhy()->GetObject<TsOokBasedNanoSpectrumPhy> ()->SetPulseDuration(
					FemtoSeconds(pulseDuration));
			dev->GetPhy()->GetObject<TsOokBasedNanoSpectrumPhy> ()->SetPulseInterval(
					FemtoSeconds(pulseInterval));
			std::cout << "Created NanoNode " << dev->GetNode()->GetId()
					<< " position " << " position " << m->GetPosition().x << " "
					<< m->GetPosition().y << " " << m->GetPosition().z << std::endl;
			dev -> m_txrange = txrange;
			dev -> m_gates = gate;
			dev -> m_routers = router;
			dev -> m_seed = SEED;
			dev -> m_nodes = NB_NODES;
			dev -> m_apps = NB_APPS;
	}


	//vytvoreni zbylych nano-stroju, ktere se volne pohybuji v prosotru
	for (uint32_t i = 500; i < d_nodes.GetN(); i++) {
		Ptr<MobilityModel> m = n_nodes.Get(i)->GetObject<MobilityModel> ();
		nano.AddMobility(
				d_nodes.Get(i)->GetObject<NanoNodeDevice> ()->GetPhy(), m);
		Ptr<NanoNodeDevice> dev = d_nodes.Get(i)->GetObject<NanoNodeDevice> ();

		Ptr<NanoRoutingEntity> routing;
		if (ROUTINGPROTOCOL == 1) {
			Ptr<FloodingNanoRoutingEntity> routing2 = CreateObject<
					FloodingNanoRoutingEntity> ();
			routing = routing2;
		} else if (ROUTINGPROTOCOL == 2) {
			Ptr<RandomNanoRoutingEntity> routing2 = CreateObject<
					RandomNanoRoutingEntity> ();
			routing = routing2;
		} else {
		}
		routing->SetDevice(dev);
		dev->SetL3(routing);

		Ptr<NanoMacEntity> mac;
		if (MACPROTOCOL == 1) {
			Ptr<TransparentNanoMacEntity> mac2 = CreateObject<
					TransparentNanoMacEntity> ();
			mac = mac2;
		} else if (MACPROTOCOL == 2) {
			Ptr<BackoffBasedNanoMacEntity> mac2 = CreateObject<
					BackoffBasedNanoMacEntity> ();
			mac = mac2;
		} else {
		}
		mac->SetDevice(dev);
		dev->SetMac(mac);

		dev->GetPhy()->SetTransmissionRange(txrange);
		dev->GetPhy()->SetTxPower(powerTransmission);
		dev->GetPhy()->GetObject<TsOokBasedNanoSpectrumPhy> ()->SetPulseDuration(
				FemtoSeconds(pulseDuration));
		dev->GetPhy()->GetObject<TsOokBasedNanoSpectrumPhy> ()->SetPulseInterval(
				FemtoSeconds(pulseInterval));

		std::cout << "Created NanoNode " << dev->GetNode()->GetId()
				<< " position " << " position " << m->GetPosition().x << " "
				<< m->GetPosition().y << " " << m->GetPosition().z << std::endl;

		dev -> m_txrange = txrange;
		dev -> m_gates = gate;
		dev -> m_routers = router;
		dev -> m_seed = SEED;
		dev -> m_nodes = NB_NODES;
		dev -> m_apps = NB_APPS;
	}

	if (APPS == nodes)
		APPS--;
	//vytvareni paketu pro vysilajici nano-stroje
	for (int i = 0; i < 500; i++) {
		Ptr<MessageProcessUnit> mpu = CreateObject<MessageProcessUnit> ();
		mpu->SetDevice(d_nodes.Get(i)->GetObject<SimpleNanoDevice> ());
		d_nodes.Get(i)->GetObject<SimpleNanoDevice> ()->SetMessageProcessUnit(
				mpu);
		mpu -> m_txrange = txrange;
		mpu -> m_gates = gate;
		mpu -> m_routers = router;
		mpu -> m_seed = SEED;
		mpu -> m_nodes = NB_NODES;
		mpu -> m_apps = NB_APPS;
		mpu -> m_interarrivalTime = packetInterval;

		double startTime = random.GetValue(0.0, 0.1);
		Simulator::Schedule(Seconds(startTime),
				&MessageProcessUnit::CreteMessage, mpu);
	}

	for (int i = 0; i < gate; i++) {

		Ptr<MessageProcessUnit> mpu = CreateObject<MessageProcessUnit> ();
		mpu->SetDevice(d_interfaces.Get(i)->GetObject<SimpleNanoDevice> ());
		d_interfaces.Get(i)->GetObject<SimpleNanoDevice> ()->SetMessageProcessUnit(
				mpu);
		mpu -> m_txrange = txrange;
		mpu -> m_gates = gate;
		mpu -> m_routers = router;
		mpu -> m_seed = SEED;
		mpu -> m_nodes = NB_NODES;
		mpu -> m_apps = NB_APPS;
		mpu->SetDevice(d_interfaces.Get(i)->GetObject<SimpleNanoDevice> ());
		d_interfaces.Get(i)->GetObject<SimpleNanoDevice> ()->SetMessageProcessUnit(
				mpu);
	}

	Simulator::Stop(Seconds(duration));
	Simulator::Run();
	Simulator::Destroy();
	Simulator::Schedule(Seconds(duration - 0.1), &PrintMemoryUsage);
}

